import numpy as np
from scipy.integrate import simps
import matplotlib.pyplot as plt
import time
import pandas as pd

start = time.time()

def find_nearest(array, value):
    array = np.asarray(array)
    dif = array - value
    AA = []
    for idx in np.arange(len(dif)):
        if dif[idx] >= 0:
            AA.append(idx)
            break
    return AA[0]

#INPUT
L = 0.875
t = 0.125
M_step = 1 / 8 * 1 * (2 * L) ** 2
dx = L/12
dq = 32.590613476014 / (M_step*200)
q_steps = 200
support = 'hinged'

#Moment curvature
df = pd.read_excel('M-kappa experiment 1.xlsx')
curv = df.kappa_xx_EPR4
mom = df.M_xx_EPR4
curv_factor = np.zeros(len(mom))
curv_crack_pos = 0.00182865530158244
mom_crack_pos = 6.41924127948405
curv_crack_neg = -0.00182865530158244
mom_crack_neg = -6.41924127948405
curv_yield_pos = 0.0402304166348138
mom_yield_pos = 32.2391075005862
curv_yield_neg = -0.0402304166348138
mom_yield_neg = -32.2391075005862

EIin = mom_crack_neg / curv_crack_neg
max_energy = 13000

for i in np.arange(len(mom)-1):
    if 2 * ((mom[i + 1] - mom[i]) / (curv[i + 1] - curv[i])) / EIin < 1.9:
        curv_factor[i + 1] = min(((mom[i + 1] - mom[i]) / (curv[i + 1] - curv[i])) / EIin, 2)
    else:
        curv_factor[i + 1] = min(2 * ((mom[i + 1] - mom[i]) / (curv[i + 1] - curv[i])) / EIin, 2)

#Discretization
nx = int(L/dx)+1
qindex = range(q_steps)
iterations = 100000000
tol = 0.0000001

#Empty lists
w_step = np.zeros(nx + 3)
Mfunction_step = np.zeros(nx)
x = np.linspace(0, L, nx)
u = []
KM = []
KL = []
increase = []
Msup = []
Mmid = []
E_FU = []
E_Mk = []
KM_FU = []
KM_Mk = []
plastic = []
q_total = []
idx_mid = 3690
print(curv_factor[idx_mid])
w = np.zeros([len(qindex), nx + 3])
Mfunction = np.zeros([len(qindex), nx])
kappa_midspan = []
Dxx = EIin * np.ones(nx)
midspan_stage = 'uncracked'
support_stage = 'uncracked'

for l in qindex:
    for k in range(iterations):
        for i in np.arange(0, nx+3):
            if i == 0:
                if support == 'hinged':
                    w_step[i] = -w_step[i + 2]
                elif support == 'clamped':
                    w_step[i] = -w_step[i + 2] + curv_factor[idx_sup] * w_step[i + 2]
            elif i == 1:
                w_step[i] = 0
            elif i == nx+1:
                w_step[i] = 2 * w_step[i - 1] - w_step[i - 2] + curv_factor[idx_mid] * (w_step[i - 2] - w_step[i - 1])
            elif i == nx+2:
                w_step[i] = 2 * w_step[i - 1] - 2 * w_step[i - 3] + w_step[i - 4]
            else:
                w_step[i] = 1/6 * (dq * dx ** 4 / Dxx[i-1] - w_step[i - 2] + 4 * w_step[i - 1] + 4 * w_step[i + 1] - w_step[i + 2])

        increase.append(w_step[nx])
        if k>100 and abs(increase[k]/increase[k-1]) < 1+tol:
            break

    if support == 'hinged':
        Msupport = 0
    elif support == 'clamped':
        Msupport = np.min(Dxx[1::]) * (w_step[2] - 2 * w_step[1] + w_step[0]) / dx ** 2
    print(Dxx[-1] * (w_step[-2] - 2 * w_step[-3] + w_step[-4]) / dx ** 2)
    Mmidspan = -1 / 8 * dq * (2 * L) ** 2
    #Mmidspan = -5.034738788359546 * dq
    Mfunction_step = -(Msupport - Mmidspan) * 4 * x * ((2 * L) - x) / (2 * L) ** 2 + Msupport

    w[0] = w_step
    Mfunction[0] = Mfunction_step
    if l > 0:
        w[l] = w_step
        w[l] += w[l - 1]
        Mfunction[l] = Mfunction_step
        Mfunction[l] += Mfunction[l - 1]

    #kappa = np.clip(np.interp(Mfunction, mom, curv), curv_yield_neg, curv_yield_pos)
    kappa = np.interp(Mfunction, mom, curv)
    #Dxx = (Mfunction[l] + 0.000001) / (kappa[l] + 0.00000000000001)
    Dxx = np.clip((Mfunction[l]+0.000001) / (kappa[l]+0.00000000000001), mom_yield_neg/curv_yield_neg, mom_crack_neg/curv_crack_neg)

    if Mfunction[l][0] < mom_crack_pos:
        support_stage = 'uncracked'
    elif Mfunction[l][0] < mom_yield_pos:
        support_stage = 'cracked'
    else:
        support_stage = 'plastic hinge'

    if Mfunction[l][-1] > mom_crack_neg:
        midspan_stage = 'uncracked'
    elif Mfunction[l][-1] > mom_yield_neg:
        midspan_stage = 'cracked'
    else:
        midspan_stage = 'plastic hinge'

    print('progress:', str(round((l + 1) / len(qindex) * 100, 2)) + '%')
    print('stage at support:', support_stage)
    print('stage at midspan:', midspan_stage)
    print('M =', Mfunction[l][-1], 'EI =', Dxx[-1])
    print(curv_factor[idx_mid], k)
    print('w_step =', w_step[nx])
    idx_sup = find_nearest(mom, Mfunction[l][0])
    idx_mid = find_nearest(mom, Mfunction[l][-1])

    # Energy based on force-displacement graph and moment-curvature graph per step
    E_FU.append(dq * (l+1) * simps(w[l][1:nx + 1], x))
    E_Mk.append(simps(kappa[l] * Mfunction[l], x))

    #Load factor and mass factor
    w_norm = (1 / w[l][nx]) * w[l][1:nx+1]
    w_norm2 = w_norm**2
    KL.append(simps(w_norm, x) / L)
    KM.append(simps(w_norm2, x) / L)

    #Displacement at mid-span
    u.append(w_step[nx])
    Msup.append(Msupport)
    Mmid.append(Mmidspan)
    kappa_midspan.append((w_step[-2] - 2 * w_step[-3] + w_step[-4]) / dx ** 2)

    if E_FU[-1] > max_energy:
        break

u = np.cumsum(u)
Msup = np.cumsum(Msup)
Mmid = np.cumsum(Mmid)
kappa_midspan = np.cumsum(kappa_midspan)
F = np.arange(dq,dq*(len(qindex)+1),dq)*2*L


print([0] + list(u))
print([0] + list(F))
print([KL[0]] + list(KL))
print([KM[0]] + list(KM))
end = time.time()
print('Computation time =', end - start, 'seconds')

#DIANA
fig, axs = plt.subplots(3, 3)
axs[0,0].plot(x, -w[-1][1:nx+1])
axs[0,0].set_title('Displacement')
axs[0,0].set_xlabel('x [m]')
axs[0,0].set_ylabel('w [m]')

axs[0,1].plot(u, F)
axs[0,1].plot(u, F * KL)
#axs[0,1].plot(df.u, df.F)
axs[0,1].set_title('F-U')
axs[0,1].set_xlabel('w [m]')
axs[0,1].set_ylabel('q [kN/m]')

axs[0,2].plot(F, KM)
axs[0,2].plot(F, KL)
axs[0,2].set_title('Mass factor')
axs[0,2].set_xlabel('q [kN/m]')
axs[0,2].set_ylabel('w [m]')

axs[1,0].plot(x, Dxx)
axs[1,0].set_title('Moment-curvature')
axs[1,0].set_xlabel('kappa [1/m]')
axs[1,0].set_ylabel('M [kNm]')

axs[1,1].plot(x, kappa[-1])
axs[1,1].set_title('Curvature')
axs[1,1].set_xlabel('x [m]')
axs[1,1].set_ylabel('kappa [1/m]')

axs[1,2].plot(x, Mfunction[-1])
axs[1,2].set_title('Bending moment')
axs[1,2].set_xlabel('x [m]')
axs[1,2].set_ylabel('M [kNm]')

axs[2,0].plot(curv[2:7],mom[2:7])
axs[2,0].set_title('Moment-curvature')
axs[2,0].set_xlabel('kappa [1/m]')
axs[2,0].set_ylabel('M [kNm]')

axs[2,1].plot(F, E_FU)
axs[2,1].plot(F, E_Mk)
axs[2,1].set_title('Work')
axs[2,1].set_xlabel('q [kN/m]')
axs[2,1].set_ylabel('energy [J]')

axs[2,2].plot(F, Msup)
#axs[2,2].plot(F, 1/12 * np.arange(dq,dq*(len(qindex)+1),dq) * (2*L)**2)
axs[2,2].plot(F, Mmid)
#axs[2,2].plot(F, -1/24 * np.arange(dq,dq*(len(qindex)+1),dq) * (2*L)**2)

axs[2,2].set_title('Work')
axs[2,2].set_xlabel('q [kN/m]')
axs[2,2].set_ylabel('energy [J]')

plt.show()


