import numpy as np
from scipy.interpolate import InterpolatedUnivariateSpline
from scipy.optimize import brentq as find_root
import matplotlib.pyplot as plt

def strain_to_stress(eps):
    stress = []
    for i in eps:
        if i > 0:
            stress.append(np.interp(i, Strain_concrete_tension, Stress_concrete_tension))
        elif i < 0:
            stress.append(np.interp(i, Strain_concrete_compression[::-1], Stress_concrete_compression[::-1]))
        else:
            stress.append(0)
    return stress
def first(the_iterable, condition = lambda x: True):
    for i in the_iterable:
        if condition(i):
            return i

############################ GEOMETRY ############################
height = 220
t = 220
height_beam = height-t
width = 1000
width_beam = 0
Ac = width * t + (height-t)*width_beam
As_bottom = 5*1/4*3.1415*8**2
As_middle = 0
As_top = 4*1/4*3.1415*6**2
z_bottom = 30 + 4
z_middle = 0
z_top = 220 - 30 - 3

############################ MATERIALS ############################
Es = 200000
ref_strain_rate = 0.001
JC_A = 571
JC_B = 643
JC_C = 0.02139
JC_n = 0.72
strain_rate = 0.03 #0.0002
yield_stress = JC_A*(1 + JC_C*np.log(strain_rate / ref_strain_rate))
yield_strain = yield_stress / Es
ultimate_strain = 0.05
ultimate_plastic_strain = ultimate_strain - yield_strain
true_ultimate_plastic_strain = np.log(ultimate_plastic_strain + 1)

True_strain_reinforcement = np.linspace(0, true_ultimate_plastic_strain, 51)
True_stress_reinforcement = (JC_A + JC_B*True_strain_reinforcement**JC_n)*(1 + JC_C*np.log(strain_rate / ref_strain_rate))
Strain_reinforcement = np.exp(True_strain_reinforcement) - 1
Stress_reinforcement = True_stress_reinforcement / (Strain_reinforcement + 1)

Strain_reinforcement = list(-Strain_reinforcement[::-1] - yield_strain) + [0] + list(Strain_reinforcement + yield_strain)
Stress_reinforcement = list(-Stress_reinforcement[::-1]) + [0] + list(Stress_reinforcement)

Gf = 0.133
ft = 2.21
E = 29962 * 1.2
h_eq = (6350/2)/12
GC = 0.133 * 250 * 1.18
fc = 28 * 1.18

def hordijk(h_eq, Gf, ft, E):
    Gf = Gf
    ft = ft
    E = E
    dun_ult = 5.136*Gf/(ft*h_eq)
    c1 = 3
    c2 = 6.93
    du = np.arange(0, dun_ult, dun_ult / 100)
    f_tension=[]

    for n in range(len(du)):
        if 0 < du[n] < dun_ult:
            f_tension.append(((1 + (c1 * du[n] / dun_ult) ** 3) * np.exp(-c2 * du[n] / dun_ult) - du[n] / dun_ult * (1 + c1 ** 3) * np.exp(-c2)) * ft)
        else:
            f_tension.append(0)

    f_tension[0] = ft
    f_tension = list(f_tension)
    f_tension = [0] + f_tension + [0]
    du = du + ft / E
    du = list(du)
    du = [0] + du + [1000*dun_ult]
    return du, f_tension
def parabolic(h_eq, GC, fc, E):
    E = E
    h_eq = h_eq
    GC = GC
    fc = fc
    alpha_c3 = -1/3 * fc/E
    alpha_c = -5/3 * fc/E
    alpha_u = alpha_c - 3/2 * GC/(h_eq*fc)
    f_compression = []
    alpha = np.arange(alpha_u, -alpha_c3/10,  -alpha_c3/10)
    alpha = alpha[::-1]

    for j in range(len(alpha)):
        if alpha_c3 < alpha[j] <= 0:
            f_compression.append(-fc/3 * alpha[j] / alpha_c3)
        elif alpha_c < alpha[j] <= alpha_c3:
            f_compression.append(-fc/3*(1 + 4*(alpha[j] - alpha_c3) / (alpha_c - alpha_c3) - 2*((alpha[j] - alpha_c3) / (alpha_c - alpha_c3))**2))
        elif alpha_u < alpha[j] <= alpha_c:
            f_compression.append(-fc*(1 - ((alpha[j] - alpha_c) / (alpha_u - alpha_c))**2))
        else:
            f_compression.append(0)

    alpha = list(alpha)
    f_compression = list(f_compression)
    alpha.append(100*alpha_u)
    f_compression.append(0)
    return alpha, f_compression

Strain_concrete_compression = parabolic(h_eq, GC, fc, E)[0]
Stress_concrete_compression = parabolic(h_eq, GC, fc, E)[1]
Strain_concrete_tension = hordijk(h_eq, Gf, ft, E)[0]
Stress_concrete_tension = hordijk(h_eq, Gf, ft, E)[1]

########### OTHER INPUT ##############
points = 71
delta_height = height/(points-1)
tol = 0.01
z_initial = 0.45*height          #initial guess of z_neutral to speed up the iterations
delta_kappa = 0.5 * Strain_concrete_tension[1] / (height / 2)
n_kappa_steps = 3000
z_increase = height / 5000
z_decrease = -z_increase

################# INITIAL LISTS AND ARRAYS #####################
kappa_array = np.arange(delta_kappa, (n_kappa_steps+1)*delta_kappa, delta_kappa)
z_neutral = z_initial*np.ones(len(kappa_array))
concrete_tensile_force = np.zeros(len(kappa_array))
concrete_compressive_force = np.zeros(len(kappa_array))
bottom_reinforcement_force = np.zeros(len(kappa_array))
middle_reinforcement_force = np.zeros(len(kappa_array))
top_reinforcement_force = np.zeros(len(kappa_array))
eps_bottom = z_neutral[0] * delta_kappa
eps_top = -(height - z_neutral[0]) * delta_kappa
delta_eps = (eps_bottom - eps_top) / (points - 1)
eps = np.linspace(eps_bottom, eps_top, points)
stress = strain_to_stress(eps)
h = np.arange(0, height+delta_height, delta_height)
W = width*np.ones(len(h))
W[h <= height - t] = width_beam
A = h * W
M_array = []
eps_top_array = []
eps_bottom_array = []
W[0] = width
print(W)
################ CALCULATION #################
for kappa in kappa_array:
    converged = False
    idx = int(round(kappa / delta_kappa) - 1)
    z_neutral[idx] = 0.93*z_neutral[idx-1]               #first guess of z_neutral based on the previous one
    while not converged:                                #for loop might be faster, but while loop makes script clearer for later use
        z_neutral[idx] += z_increase
        eps_top = -(height - z_neutral[idx]) * kappa
        eps_bottom = z_neutral[idx] * kappa
        delta_eps = (eps_bottom - eps_top) / (points - 1)
        eps = np.linspace(eps_bottom, eps_top, points)
        stress = strain_to_stress(eps)
        f = InterpolatedUnivariateSpline(h, stress, k=1)

        if z_neutral[idx] >= height_beam:
            Ac_under_compression = (height-z_neutral[idx]) * width
            Ac_under_tension = Ac - Ac_under_compression
        else:
            Ac_under_tension = z_neutral[idx] * width_beam
            Ac_under_compression = Ac - Ac_under_tension

        concrete_tensile_force[idx] = f.integral(0, z_neutral[idx]) / z_neutral[idx] * Ac_under_tension
        concrete_compressive_force[idx] = f.integral(z_neutral[idx], height) / (height - z_neutral[idx]) * Ac_under_compression

        bottom_reinforcement_strain = np.interp(z_bottom, h, eps)
        bottom_reinforcement_force[idx] = np.interp(bottom_reinforcement_strain, Strain_reinforcement, Stress_reinforcement) * As_bottom
        middle_reinforcement_strain = np.interp(z_middle, h, eps)
        middle_reinforcement_force[idx] = np.interp(middle_reinforcement_strain, Strain_reinforcement, Stress_reinforcement) * As_middle
        top_reinforcement_strain = np.interp(z_top, h, eps)
        top_reinforcement_force[idx] = np.interp(top_reinforcement_strain, Strain_reinforcement, Stress_reinforcement) * As_top

        if concrete_tensile_force[idx] + concrete_compressive_force[idx] + top_reinforcement_force[idx] + middle_reinforcement_force[idx] + bottom_reinforcement_force[idx] < 0:
            converged = False
        else:
            converged = True

    z = InterpolatedUnivariateSpline(h, stress * W * (z_neutral[idx]-h), k=1)
    M_concrete = z.integral(0, height)

    z_bottom_reinforcement = z_neutral[idx] - z_bottom
    z_middle_reinforcement = z_neutral[idx] - z_middle
    z_top_reinforcement = z_neutral[idx] - z_top

    M = abs(M_concrete) + abs(bottom_reinforcement_force[idx] * z_bottom_reinforcement) + abs(middle_reinforcement_force[idx] * z_middle_reinforcement) + abs(top_reinforcement_force[idx] * z_top_reinforcement)
    M_array.append(M)
    eps_top_array.append(eps_top)
    eps_bottom_array.append(eps_bottom)
    print('step:', idx + 1, '/', n_kappa_steps)
    print('residual force:', concrete_compressive_force[idx] + concrete_tensile_force[idx] + bottom_reinforcement_force[idx] + top_reinforcement_force[idx] + middle_reinforcement_force[idx], 'N')
    print('bottom reinforcement stress', round(bottom_reinforcement_force[idx] / As_bottom, 2), 'MPa')
    print('top reinforcement stress', round(top_reinforcement_force[idx] / As_top, 2), 'MPa')
    print('z-neutral:', z_neutral[idx], 'mm')
    print('kappa:', kappa, '1/mm')
    print('M:', round(M,2), 'Nmm')
    print('-----------------------------------------------------------------------')


M_array = [0] + M_array

height_line = [[0, 0],
               [0, height]]
neutral_line_strain = [[eps_top, eps_bottom],
                       [z_neutral[-1], z_neutral[-1]]]
neutral_line_stress = [[min(stress), max(stress)],
                       [z_neutral[-1], z_neutral[-1]]]

fig, axs = plt.subplots(2,2)
axs[0,0].plot(eps, h)
axs[0,0].plot(neutral_line_strain[0], neutral_line_strain[1], linestyle='dashed', color='r')
axs[0,0].plot(height_line[0], height_line[1], color='k')
axs[0,0].set_title('Strain (final stage)')
axs[0,0].set_xlabel('eps [-]')
axs[0,0].set_ylabel('h [mm]')

axs[0,1].plot(stress, h)
axs[0,1].plot(neutral_line_stress[0], neutral_line_stress[1], linestyle='dashed', color='r')
axs[0,1].plot(height_line[0], height_line[1], color='k')
axs[0,1].set_title('Stress in concrete (final stage)')
axs[0,1].set_xlabel('sigma [MPa]')
axs[0,1].set_ylabel('h [mm]')

axs[1,0].plot([0]+list(kappa_array), M_array)
axs[1,0].set_title('Moment-curvature')
axs[1,0].set_xlabel('kappa [1/m]')
axs[1,0].set_ylabel('M [kNm]')

top_reinforcement_line = [[z_top, z_top],
                       [top_reinforcement_force[-1] / As_top, 0]]
middle_reinforcement_line = [[z_middle, z_middle],
                       [middle_reinforcement_force[-1] / As_middle, 0]]
bottom_reinforcement_line = [[z_bottom, z_bottom],
                       [bottom_reinforcement_force[-1] / As_bottom, 0]]

axs[1,1].plot(top_reinforcement_line[1], top_reinforcement_line[0])
axs[1,1].plot(middle_reinforcement_line[1], middle_reinforcement_line[0])
axs[1,1].plot(bottom_reinforcement_line[1], bottom_reinforcement_line[0])
axs[1,1].plot(height_line[0], height_line[1], color='k')
axs[1,1].set_title('Stress in reinforcement (final stage)')
axs[1,1].set_xlabel('sigma [MPa]')
axs[1,1].set_ylabel('h [mm]')
plt.show()

print([0]+list(kappa_array))
print(M_array)